@@ -34,7 +34,7 @@
 #define _GLIBCXX_RELEASE 11
 
 // The datestamp of the C++ library in compressed ISO date format.
-#define __GLIBCXX__ 20230528
+#define __GLIBCXX__ 20240719
 
 // Macros for various attributes.
 //   _GLIBCXX_PURE
@@ -848,6 +848,9 @@ namespace std
 /* Define to 1 if you have the <dirent.h> header file. */
 #define _GLIBCXX_HAVE_DIRENT_H 1
 
+/* Define if dirfd is available in <dirent.h>. */
+#define _GLIBCXX_HAVE_DIRFD 1
+
 /* Define to 1 if you have the <dlfcn.h> header file. */
 #define _GLIBCXX_HAVE_DLFCN_H 1
 
@@ -875,6 +878,9 @@ namespace std
 /* Define to 1 if you have the <fcntl.h> header file. */
 #define _GLIBCXX_HAVE_FCNTL_H 1
 
+/* Define if fdopendir is available in <dirent.h>. */
+#define _GLIBCXX_HAVE_FDOPENDIR 1
+
 /* Define to 1 if you have the <fenv.h> header file. */
 #define _GLIBCXX_HAVE_FENV_H 1
 
@@ -1064,6 +1070,9 @@ namespace std
 /* Define if <math.h> defines obsolete isnan function. */
 /* #undef _GLIBCXX_HAVE_OBSOLETE_ISNAN */
 
+/* Define if openat is available in <fcntl.h>. */
+#define _GLIBCXX_HAVE_OPENAT 1
+
 /* Define if poll is available in <poll.h>. */
 #define _GLIBCXX_HAVE_POLL 1
 
@@ -1253,11 +1262,14 @@ namespace std
 #define _GLIBCXX_HAVE_TRUNCATE 1
 
 /* Define to 1 if you have the <uchar.h> header file. */
-/* #undef _GLIBCXX_HAVE_UCHAR_H */
+#define _GLIBCXX_HAVE_UCHAR_H 1
 
 /* Define to 1 if you have the <unistd.h> header file. */
 #define _GLIBCXX_HAVE_UNISTD_H 1
 
+/* Define if unlinkat is available in <fcntl.h>. */
+#define _GLIBCXX_HAVE_UNLINKAT 1
+
 /* Define to 1 if you have the `uselocale' function. */
 #define _GLIBCXX_HAVE_USELOCALE 1
 
@@ -1656,7 +1668,7 @@ namespace std
 
 /* Define if C11 functions in <uchar.h> should be imported into namespace std
    in <cuchar>. */
-/* #undef _GLIBCXX_USE_C11_UCHAR_CXX11 */
+#define _GLIBCXX_USE_C11_UCHAR_CXX11 1
 
 /* Define if C99 functions or macros from <wchar.h>, <math.h>, <complex.h>,
    <stdio.h>, and <stdlib.h> can be used or exposed. */
